package orchid.agent;

import orchid.server.*;
import orchid.object.*;

import java.io.*;
import java.util.*;
import com.objectspace.voyager.*;
import com.objectspace.voyager.space.*;
import com.objectspace.voyager.space.Subspace.*;
import com.objectspace.voyager.space.multicasting.*;
import com.objectspace.lib.*;
import com.objectspace.lib.util.*;
import com.objectspace.voyager.agent.*;
import java.math.*;
import java.lang.Math.*;

/** Server Administrator Agent Thread class
  * @author Antoine SOULIEZ
  */

public class Saa
  {
    private static ISubspace selfserverspace;
    private static ISubspace globalserverspace;
    private static ISubspace selfagentspace;
    private static ISubspace globalagentspace;

		private SaaConfig saacfg;
		private ITracer tracer;
		private boolean tracerison = false;


    private boolean masterson = false;
		private ISaa mastersaa;

		private ServerInfo servinfo;

    private String strtodisp;

    private Hashtable loggedusers = new Hashtable();

	public Saa(SaaConfig saacfg)
		{
			this.saacfg = saacfg;
      masterson =  (saacfg.MASTERSERVER.equals(saacfg.selfID));
			servinfo = new ServerInfo("orchid-"+saacfg.selfID, saacfg.selfID, 10, masterson);
		}

  /** Initialize the SAA
    */
  private void saaInit()
    {
      try
        {
          selfserverspace = (ISubspace) Namespace.lookup("//"+saacfg.selfID+"/"+saacfg.SELFSERVERSPACENAME);
          selfagentspace = (ISubspace) Namespace.lookup("//"+saacfg.selfID+"/"+saacfg.SELFAGENTSPACENAME);
          cMessage("info","self subspaces reached");
        } catch (NamespaceException nsexception)
          {
            cMessage("error","Unable to find server self subspaces >> "+nsexception.toString());
            System.exit(1);
          }

      try
        {
          globalserverspace = (ISubspace) Namespace.lookup("//"+saacfg.MASTERSERVER+"/"+saacfg.GLOBALSERVERSPACENAME);
          globalagentspace = (ISubspace) Namespace.lookup("//"+saacfg.MASTERSERVER+"/"+saacfg.GLOBALAGENTSPACENAME);
          cMessage("info","global subspaces reached");
        } catch (Exception nsexception2)
          {
            cMessage("error","Unable to find global subspaces >> "+nsexception2.toString());
            System.exit(1);
          }

        try
          {
            ISaa selfsaa = (ISaa) Namespace.lookup("//"+saacfg.selfID+"/SAA");
            selfagentspace.add(selfsaa);
            globalagentspace.add(selfsaa);
            cMessage("info","Added to Agents subspaces");
          } catch (Exception e1)
            {
              cMessage("error","Can't add Saa to Agents Subspaces >> "+e1.toString());
              System.exit(1);
            }

				try
					{
						mastersaa = (ISaa) Namespace.lookup("//"+saacfg.MASTERSERVER+"/SAA");
					} catch (Exception e3)
						{
							cMessage("error","Can't reach mastersaa >> "+e3.toString());
						}
				/* Tells the others SAA that this server is connected */
				mastersaa.setConnection(servinfo,true);

				/* Try to find the Tracer */
				try
					{
						tracer = (ITracer) Namespace.lookup("//"+saacfg.selfID+"/Tracer");
						cMessage("info","Tracer found");
						tracerison = true;
					} catch (Exception e2)
						{
							cMessage("alert","Cannot find Tracer");
						}

    }

  /** run the SAA
    */
  public void live()
    {
      saaInit();
      if (masterson)
        {
        }
      else
        {
        }
    }

  /** Disconnect the SAA
    */
  public void saaDisconnect()
    {
      cMessage("info","Disconnecting");
			mastersaa.setConnection(servinfo,false);
    }

  /** Stop the SAA
    */
  public void saaStop()
    {
      cMessage("info","Stopping");
    }

	/** Ask the Saa to create a subspace and populate it.<BR>
		* <T>spacename</T> is the name of the subspace to be created.<BR>
		* <T>proxynamelist</T> is a vector with the complete URL of each agent to include in the subspace.
		*/
	public void createSubspace(String spacename, Vector proxynamelist)
		{
			ISubspace talkroom;
			try
				{
					talkroom = (ISubspace) Factory.create("com.objectspace.voyager.space.Subspace","//"+saacfg.selfID+"/"+spacename);
					cMessage("info",spacename+" subspace created");
					Enumeration enum = proxynamelist.elements();
					while (enum.hasMoreElements())
						{
							try
								{
									IMeetingAgent ma = (IMeetingAgent) Namespace.lookup(enum.nextElement().toString());
									talkroom.add(ma);
									cMessage("info","added 1 agent to "+spacename);
								} catch (NamespaceException nse)
									{
										cMessage("error","Unable to reach "+spacename+" : "+nse);
									}
						}
				} catch (Exception e)	
				{
					cMessage("error","Unable to create subspace "+spacename+" : "+e);
				}
		}

  /** Display a signed message
    */
  public void logMessg (String purpose, String identity, String msgtodisp)
    {
      orchid.object.Util.logMsg(purpose, identity, msgtodisp);
			if (tracerison)
				tracer.logMessage(purpose, identity, msgtodisp);
    }

	private void cMessage(String purpose, String msgtodisp)
		{
			logMessg(purpose, "SAA "+saacfg.selfID, msgtodisp);
		}

  /** give a alphanumeric string if the user is authentified
    * give only numerical string for now
    */
  public String getAuthentication(String username, String password)
    {
			cMessage("info","Received: username="+username+" password="+password);
      if (saacfg.father.getAuthentication(username, password))
        {
          double randy;
          double alphanum;
          double temporiseint = 0;
          String userkey = null;

          for (int i = 1 ; i<=10 ; i++)
            {
              alphanum = java.lang.Math.random();
              randy = java.lang.Math.random();
              if (alphanum > 0.5)
                temporiseint = temporiseint*10 + (int) java.lang.Math.floor(randy*10);
              else
                temporiseint = temporiseint*10 + (int) java.lang.Math.floor(randy*10);
            }
          userkey = (new Double(temporiseint)).toString();
          loggedusers.put(userkey.toString(),username);
          cMessage("info",username+" logged in");
          return userkey;
        }
      else
        {
					cMessage("info","Attempt to log on with wrong username/password !");
          return "";
        }
    }

  public Vector getServers(String filter)
    {
      return saacfg.father.getServers(filter);
    }

  public void addServer(ServerInfo servercard)
    {
      saacfg.father.addServer(servercard);
      cMessage("info","Server "+servercard.getUrl()+" is online");
    }

  public void removeServer(ServerInfo servercard)
    {
      saacfg.father.removeServer(servercard);
      cMessage("info","Server "+servercard.getUrl()+" is offline");
    }
  
  public Vector getAgents(String filter)
    {
      return saacfg.father.getAgents(filter);
    }

  public void addAgent(AgentInfo agentinfo)
    {
      saacfg.father.addAgent(agentinfo);
      cMessage("info","Agent "+agentinfo.getCurrentlocation()+" is online");
    }
  
  public void removeAgent(AgentInfo agentinfo)
    {
      saacfg.father.removeAgent(agentinfo);
      cMessage("info","Agent "+agentinfo.getCurrentlocation()+" is offline");
    }

	public Vector getMeetings(String username, Date beginning, Date end) {
		return saacfg.father.getMeetings(username, beginning, end);
	}

	public void addMeeting(String username, Meeting m) {
		saacfg.father.addMeeting(username, m);
	}
	
  /** give the location of MA agent of the user identified by the Identificator string
    */
  public String getMALocation(String Identificator)
    {
      String username = (String) loggedusers.get(Identificator);
      if (!username.equals(""))
        {
          /* requete a la base des users */
          return "coin";
        }
      else
        {
          return "";
        }
    }
	

	public void setConnection(ServerInfo server, boolean connecting)
		{
			if (masterson)
				{
					ISaa saaslave;
					try
						{
							saaslave = (ISaa) globalagentspace.getMulticastProxy("orchid.agent.ISaa");
							if (connecting)
								saaslave.addServer(server);
							else
								saaslave.removeServer(server);
						} catch (Exception except1)
							{
								cMessage("error","Cannot get a multicast proxy over SAAs "+server.getUrl());
							}

				}
		}
  }

