package orchid.object;

import java.util.*;

/**
 * A class used to propose a meeting to people.<BR>
 * A meeting proposal looks like:<P>
 * <TT>
 * runion tped dfinition du terme agent ma_darche Mon Sep 28 00:00:00 GMT+00:00 1998 Sat Oct 10 00:00:00 GMT+00:00 1998 400 philippe_benhamou emmanuel_pierre antoine_souliez
 * </TT>
 * <P>The objects to arrange a meeting are used in this order :
 * <ol>
 * <li> MeetingProposal
 * <li> DateHourProposal
 * <li> RangeProposal
 * <li> Meeting
 * </ol>
 * @see DateHourProposal 
 * @see RangeProposal
 * @see Meeting
 * @see ObjectTest
 * @author Marc-Aurle DARCHE
 */
public class MeetingProposal implements java.io.Serializable{

	/**
	 * Key is a unique string defining a meeting, it is the ID of the meeting.
	 */
	private String key;
	private String subject;
	private String note;
	private String initiator;

	/**
	 * yyyy-mm-dd
	 */
	private Date betweenDate;
	private Date andDate;

	/**
	 * hh30 or hh00
	 */
	private int duration;
	
	//private Vector guestInfoList;
	private Hashtable guestInfoList;
	private int timeout;

	public MeetingProposal() {
	}

	public MeetingProposal(String subject, String note, String initiator, int year1, int month1, int day1, int year2, int month2, int day2, int duration, Hashtable guestInfoList, int timeout) {
		//this.key= (System.currentTime()).toString()+initiator;
		this.key= (new Long(System.currentTimeMillis())).toString()+initiator;
		this.subject=subject;
		this.note=note;
		this.initiator=initiator;
		Calendar rightNow1 = Calendar.getInstance();
		rightNow1.set(year1,month1,day1,0,0,0);
		this.betweenDate=rightNow1.getTime();
		Calendar rightNow2 = (Calendar)rightNow1.clone();
		rightNow2.set(year2,month2,day2,0,0,0);
		this.andDate=rightNow2.getTime();
		this.duration=duration;
		this.guestInfoList=guestInfoList;
		this.timeout=timeout;
	}

	public void setDate(int year1, int month1, int day1, int year2, int month2, int day2) {
		Calendar rightNow1 = Calendar.getInstance();
		rightNow1.set(year1,month1,day1);
		this.betweenDate=rightNow1.getTime();
		Calendar rightNow2 = Calendar.getInstance();
		rightNow2.set(year2,month2,day2);
		this.andDate=rightNow2.getTime();
	}

	public synchronized void setParticipantList(Hashtable participantlist) {
		this.guestInfoList = participantlist;
	}

	public synchronized void setAnswer(GuestInfo gi) {
		int temp=gi.getAnswer();
		gi.setAnswer(-1);
		guestInfoList.remove(gi.getName());
		gi.setAnswer(temp);
		guestInfoList.put(gi.getName(),gi);
		notifyAll();
	}

	public String getKey() {
		return this.key;
	}

	public String getSubject() {
		return this.subject;
	}

	public String getNote() {
		return this.note;
	}

	public String getInitiator() {
		return this.initiator;
	}

	public Date getBetweenDate() {
		return this.betweenDate;
	}

	public Date getAndDate() {
		return this.andDate;
	}

	/**
	 * Get the number of days between betweenDate and andDate.
	 */
	public synchronized int getRangeLength() {
		int rangeLength=1;
		Calendar betweenCal= Calendar.getInstance();
		Calendar andCal= (Calendar)betweenCal.clone();
		betweenCal.setTime(this.betweenDate);
		andCal.setTime(this.andDate);

		while ( betweenCal.before(andCal) ) {
			andCal.add(Calendar.DATE,-1); rangeLength++;
		}

		return rangeLength;
	}

	public int getDuration() {
		return this.duration;
	}

	public synchronized Hashtable getGuestInfoList() {
		return this.guestInfoList;
	}

	public int getTimeout() {
		return this.timeout;
	}

	public String toString() {
		return subject+" "+note+" "+initiator+" "+
			betweenDate.toString()+" "+andDate.toString()+" "+
			new Integer(duration).toString()+" "+guestInfoList+" "+timeout;
	}
	
}
