package orchid.server;

import java.io.*;
import java.net.*;
import java.util.Date;
import java.util.Vector;
import com.objectspace.voyager.*;
import com.objectspace.voyager.space.*;
import com.objectspace.voyager.space.Subspace.*;
import com.objectspace.lib.*;
import com.objectspace.lib.util.*;
import java.sql.*;

import orchid.object.*;
import orchid.agent.*;

/** Orchid server class
	* @author Antoine SOULIEZ
	*/

public class Server extends Thread
	{
		private boolean master;
		private static ISubspace selfserverspace;
		private static ISubspace globalserverspace;
		private static ISubspace selfagentspace;
		private static ISubspace globalagentspace;
		private static ISaa saa;

		private ServerConfig servercfg;

		private static ITracer tracer;
		private boolean tracerison = false;

		private DBQuerying dbqu = new DBQuerying("orchid_db");

		/** Construct an Orchid server with all the parameters
			*/
		public Server(ServerConfig servercfg)
			{
				this.servercfg = servercfg;
			}

		/** Initialize the Orchid Server
			*/
		private void serverInit()
			{
				try
					{
						Voyager.startup("//"+servercfg.serverID);
					} catch (StartupException exception1)
						{
							cMessage("error","Unable to start the Voyager server");
							System.exit(1);
						}
					cMessage("info","Voyager server started");

				master = (servercfg.serverID.equals(servercfg.MASTERSERVER));
				if (master)
					cMessage("info","Server status : Master");
				else
					cMessage("info","Server status : Slave");
					
				/* Create the selfspaces
					*/
				try
					{
						selfserverspace = (ISubspace) Factory.create("com.objectspace.voyager.space.Subspace","//"+servercfg.serverID+"/"+servercfg.SELFSERVERSPACENAME);
						selfagentspace = (ISubspace) Factory.create("com.objectspace.voyager.space.Subspace","//"+servercfg.serverID+"/"+servercfg.SELFAGENTSPACENAME);
						cMessage("info","self subspaces created");
					} catch (Exception exception21)
						{
							cMessage("error","Unable to create self subspaces >> "+exception21.toString());
							System.exit(1);
						}

				try
					{
						tracer = (ITracer) Factory.create("orchid.server.Tracer",new Object[]{servercfg.logpath, servercfg.serverID},"//"+servercfg.serverID+"/Tracer");
						tracerison = true;
					} catch (Exception exception33)
						{
							cMessage("error","Unable to create Tracer >> "+exception33.toString());
						}

				if (master)
					{
						try
							{
								globalserverspace = (ISubspace) Factory.create("com.objectspace.voyager.space.Subspace","//"+servercfg.serverID+"/"+servercfg.GLOBALSERVERSPACENAME);
								globalagentspace = (ISubspace) Factory.create("com.objectspace.voyager.space.Subspace","//"+servercfg.serverID+"/"+servercfg.GLOBALAGENTSPACENAME);
								cMessage("info","global subspaces created");
							} catch (Exception exception22)
								{
							cMessage("error","Unable to create global subspaces >> "+exception22.toString());
							System.exit(1);
								}
					}


				/* Create the SAA of the server
					*/
				try
					{
						SaaConfig saacfg = new SaaConfig(servercfg.serverID, servercfg.MASTERSERVER, servercfg.SELFSERVERSPACENAME, servercfg.GLOBALSERVERSPACENAME, servercfg.SELFAGENTSPACENAME, servercfg.GLOBALAGENTSPACENAME, this);
						Object[] saacfgobj = new Object[]{saacfg};

						saa = (ISaa) Factory.create("orchid.agent.Saa",saacfgobj,"//"+servercfg.serverID+"/SAA");
						cMessage("info","SAA started");
						System.out.println("-*-*-*-*-");
					} catch (Exception exception3)
						{
							cMessage("error","Unable to start SAA >> "+exception3.toString());
							System.exit(1);
						}
				saa.live();
			}

		public void run()
			{
				System.out.println("\n*** Starting Orchid Server ***\n");
				serverInit();
			}

		public void serverDisconnect()
			{
				System.out.println("\n*** Ending Orchid Server ***\n");
				cMessage("info","Disconnecting");
				saa.saaDisconnect();
			}

		public void serverStop()
			{
				cMessage("info","Stopping");
				saa.saaStop();
				try { this.sleep(1000);} catch (InterruptedException ee) {};
				tracer.die();
				System.out.println("\n*** End of Orchid Server ***\n");
				System.exit(0);
			}

		/*
			******************** Database requests *******************
		*/
		public boolean getAuthentication(String username, String password)
			{
				return dbqu.getAuthentication(username, password);
			}

		/* Server purpose requests */
		public Vector getServers(String filter)
			{
				return dbqu.getServers(filter);
			}

		public void addServer(ServerInfo serverID)
			{
				dbqu.addServer(serverID);
			}

		public void removeServer(ServerInfo serverID)
			{
				dbqu.removeServer(serverID);
			}

		/* Agent purpose requests */
		public Vector getAgents(String filter)
			{
				return dbqu.getAgents(filter);
			}

		public void addAgent(AgentInfo agentID)
			{
				dbqu.addAgent(agentID);
			}

		public void removeAgent(AgentInfo agentID)
			{
				dbqu.removeAgent(agentID);
			}


		/* Meeting purpose requests */
		public Vector getMeetings(String username, Date beginning, Date end)
			{
				return dbqu.getMeetings(username, beginning, end);
			}

		public void addMeeting(String user, Meeting m)
			{
				dbqu.addMeeting(user, m);
			}

		/* message log */
		private void logMessg(String purpose, String identity, String message)
			{
				orchid.object.Util.logMsg(purpose, identity, message);
				if (tracerison)
					tracer.logMessage(purpose, identity, message);
			}

		private void cMessage(String purpose, String msgtodisp)
			{
				logMessg(purpose, "server "+servercfg.serverID, msgtodisp);
			}
		
	}
 
